package come.stormborntales.notevault

import android.net.Uri
import android.os.Bundle
import android.util.Log
import androidx.activity.ComponentActivity
import androidx.activity.compose.rememberLauncherForActivityResult
import androidx.activity.compose.setContent
import androidx.activity.result.contract.ActivityResultContracts
import androidx.compose.foundation.layout.padding
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.Menu
import androidx.compose.material3.DrawerValue
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.Icon
import androidx.compose.material3.IconButton
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.ModalDrawerSheet
import androidx.compose.material3.ModalNavigationDrawer
import androidx.compose.material3.NavigationDrawerItem
import androidx.compose.material3.NavigationDrawerItemDefaults
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Text
import androidx.compose.material3.TopAppBar
import androidx.compose.material3.rememberDrawerState
import androidx.compose.runtime.*
import androidx.compose.ui.Modifier
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.unit.dp
import androidx.lifecycle.viewmodel.compose.viewModel
import androidx.navigation.compose.NavHost
import androidx.navigation.compose.composable
import androidx.navigation.compose.rememberNavController
import come.stormborntales.notevault.data.local.AppDatabase
import come.stormborntales.notevault.data.local.entity.NoteEntity
import come.stormborntales.notevault.data.repository.NoteRepository
import come.stormborntales.notevault.ui.screens.AddNoteDialog
import come.stormborntales.notevault.ui.screens.MainScreen
import come.stormborntales.notevault.ui.screens.SettingsScreen
import come.stormborntales.notevault.ui.theme.NoteVaultTheme
import come.stormborntales.notevault.ui.viewmodel.NoteViewModel
import come.stormborntales.notevault.ui.viewmodel.NoteViewModelFactory
import kotlinx.coroutines.launch


class MainActivity : ComponentActivity() {
    @OptIn(ExperimentalMaterial3Api::class)
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        val applicationContext = applicationContext
        val database = AppDatabase.getDatabase(applicationContext)
        val repository = NoteRepository(database.noteDao())
        val viewModelFactory = NoteViewModelFactory(repository)

        setContent {
            NoteVaultTheme {
                val navController = rememberNavController()
                val viewModel: NoteViewModel = viewModel(factory = viewModelFactory)

                var selectedUris by remember { mutableStateOf<List<Uri>>(emptyList()) }
                var showDialog by remember { mutableStateOf(false) }
                var noteToEdit by remember { mutableStateOf<NoteEntity?>(null) }
                val drawerState = rememberDrawerState(DrawerValue.Closed)
                val scope = rememberCoroutineScope()

                val imagePickerLauncher = rememberLauncherForActivityResult(
                    contract = ActivityResultContracts.OpenMultipleDocuments(),
                    onResult = { uris ->
                        if (uris.isNotEmpty()) {
                            selectedUris = uris
                            showDialog = true
                        }
                    }
                )

                val openDialog: (NoteEntity?) -> Unit = { note ->
                    Log.d("EditNote", "NoteEntity: ${note?.title}")
                    noteToEdit = note
                    showDialog = true
                }

                val navItems = listOf(
                    "Home" to "main",
                    "Einstellungen" to "settings"
                )

                ModalNavigationDrawer(
                    drawerState = drawerState,
                    drawerContent = {
                        ModalDrawerSheet {
                            Text("Navigation", modifier = Modifier.padding(16.dp), style = MaterialTheme.typography.titleMedium)
                            navItems.forEach { (label, route) ->
                                NavigationDrawerItem(
                                    label = { Text(label) },
                                    selected = false,
                                    onClick = {
                                        navController.navigate(route) {
                                            popUpTo(navController.graph.startDestinationId) { saveState = true }
                                            launchSingleTop = true
                                            restoreState = true
                                        }
                                        scope.launch { drawerState.close() }
                                    },
                                    modifier = Modifier.padding(NavigationDrawerItemDefaults.ItemPadding)
                                )
                            }
                        }
                    }
                ) {
                    Scaffold(
                        topBar = {
                            TopAppBar(
                                title = { Text("NoteVault") },
                                navigationIcon = {
                                    IconButton(onClick = {
                                        scope.launch { drawerState.open() }
                                    }) {
                                        Icon(Icons.Default.Menu, contentDescription = "Menu")
                                    }
                                }
                            )
                        }
                    ) { innerPadding ->
                        NavHost(
                            navController = navController,
                            startDestination = "main",
                            modifier = Modifier.padding(innerPadding)
                        ) {
                            composable("main") {
                                MainScreen(
                                    viewModel = viewModel,
                                    onAddNoteClicked = {
                                        imagePickerLauncher.launch(arrayOf("image/*"))
                                    },
                                    onEditNote = openDialog
                                )
                            }
                            composable("settings") {
                                SettingsScreen()
                            }
                        }

                        if (showDialog) {
                            val context = LocalContext.current
                            AddNoteDialog(
                                onDismiss = { showDialog = false },
                                onSave = { title, composer, year, genre, description ->
                                    if (noteToEdit == null) {
                                        viewModel.addNote(
                                            context = context,
                                            title = title,
                                            composer = composer,
                                            year = year,
                                            genre = genre,
                                            description = description,
                                            selectedUris = selectedUris,
                                            onDone = { showDialog = false }
                                        )
                                    } else {
                                        viewModel.updateNote(
                                            editedNote = noteToEdit!!,
                                            updatedTitle = title,
                                            updatedComposer = composer,
                                            updatedYear = year,
                                            updatedGenre = genre,
                                            updatedDescription = description,
                                            onDone = { showDialog = false }
                                        )
                                    }
                                },
                                initialTitle = noteToEdit?.title ?: "",
                                initialComposer = noteToEdit?.composer,
                                initialYear = noteToEdit?.year,
                                initialGenre = noteToEdit?.genre,
                                initialDescription = noteToEdit?.description
                            )
                        }
                    }
                }
            }
        }


    }
}
